#!/usr/bin/perl

#
# Created by Wim Lewis <wiml@hhhh.org>
# Copyright (c) 2003
# Based on scripts (c) 2001 by Jeff Mock <jeff@mock.com>
#

use CGI::Carp qw(carpout);
use DB_File;

our($index_fn, $tracks_fn, @music, $log_fn, %dbkeys);

do 'rioserve.common';

open(LOG, ">>$log_fn") || warn "LOG to $log_fn: $!\n";
carpout(LOG);

my (
    $IX, %btree
);

if(defined($ENV{'REQUEST_URI'})) {
    my $script = $ENV{'REQUEST_URI'};
    
    $script =~ s-^/--;
    $script =~ s-[/?;#].*$--;

    if ($script eq 'content') {
	&do_emit_music();
    } else {
	require CGI;
	import CGI;

	my $q = new CGI;
	$IX = tie %btree, 'DB_File', $index_fn, O_RDONLY, 0666, $DB_BTREE;

	if ($script eq 'query') {
	    &do_query($q);
	} elsif ($script eq 'list') {
	    &do_list($q);
	} elsif ($script eq 'tags') {
	    &list_tags($q);
	} elsif ($script eq 'results') {
	    &get_results($q);
	} else {
	    &do_web_admin($q);
	}
    }
} else {
    print "Right now this only does anything useful if invoked as a CGI script.\n";
}

exit 0;

sub get_one_track_info
{
    my($trackid) = @_;
    my(%tracks, @dbfields);
    
    tie %tracks, 'DB_File', $tracks_fn, O_RDONLY, 0666, $DB_HASH;
    @dbfields = split("\000", $tracks{$trackid});
    untie %tracks;

    @dbfields;
}

sub do_query {
    my($q) = @_;

    my (
	$search,  # Search string
        $dbkey,   # Part of dbm file to search
#	%dbm,     # Tied to music database
#	$IX,      # The DB_File instance
	%results, # List of items matching query
	@resultkeys,  # Sorted list of keys to %results
	$begin,   # Beginning index to report
	$end,     # Ending index to report
    );

#
# Find search string if present in URL.
#
# The URL looks like this if there isn't there isn't a search string;
#    http://.../query?artist=
#
# If there is a search string the URL looks like:
#    http://.../query?artist=[1abc]
#
# Hence the distintion between if(defined($q->param(artist))) and 
# if ($q->param(artist)).
#

    KEY: foreach (keys %dbkeys) {
	if (defined($q->param($_))) {
	    $dbkey = $dbkeys{$_};
	    $search = $q->param($_) if ($q->param($_));
	    last KEY;
	}
    }
    die "No query parameter passed to script, stopped" unless $dbkey;

    my $itemkey = $dbkey;
    my $itemvalue = '';
    $IX->seq($itemkey, $itemvalue, R_CURSOR);
    while ( substr($itemkey, 0, 1) eq $dbkey ) {
	my $item = substr($itemkey, 1);
	
	# TODO: Searches.

	$results{$item} = 0 unless exists $results{$item};
	$results{$item} ++;

	$IX->seq($itemkey, $itemvalue, R_NEXT) && last;
    }

    @resultkeys = sort { uc($a) cmp uc($b) } keys %results;

    $begin = 0;
    if (defined($q->param('_begin'))) {
	$begin = ( 0 + $q->param('_begin') );
	$begin = $#resultkeys if $begin > $#resultkeys;
    }
    $end = $#resultkeys;
    if (defined($q->param('_end'))) {
	$end = ( 0 + $q->param('_end') );
	$end = $#resultkeys if $end > $#resultkeys;
    }
    
    # Emit the results.
    print "Content-Type: text/plain\n\n";
    print "matches=\n";
    foreach my $ix ($begin .. $end) {
	printf "%d=%d,0,0:%s\n", $ix, $results{$resultkeys[$ix]}, $resultkeys[$ix];
    }    
}  # end of do_query
		
sub get_results {
    my ($q) = @_;
    
    my ($dbkey, $search, %tracks);

    tie %tracks, 'DB_File', $tracks_fn, O_RDONLY, 0666, $DB_HASH;

    KEY: foreach (keys %dbkeys) {
	if (defined($q->param($_))) {
	    $dbkey = $dbkeys{$_};
	    $search = $q->param($_);
	    last KEY;
	}
    }
    die "No query parameter passed to script, stopped" unless $dbkey;

    my @tracks;
    foreach my $trackid ($IX->get_dup( $dbkey . $search )) {
	my @trackinfo = split("\000", $tracks{$trackid});
	push(@tracks, { 'trackid' => $trackid, @trackinfo });
    }

    my @result;
    if ($q->param('_extended') == 2) {
	# binary format
	@result = map { pack('V3',
			     hex($_->{'trackid'}),
			     $_->{$dbkeys{'filesize'}},
			     0 ) } @tracks;
    } else {
	# text format
	@result = map { sprintf("%s=T%s\n",
				$_->{'trackid'},
				$_->{$dbkeys{'title'}}) } @tracks;
    }
    
    my $result = join('', @result);

    printf "Content-Length: %d\n", length($result);
    print "Content-Type: text/plain\n\n";
    print $result;
} # end of get_results

{
my(%tag_info_tags, %tag_info_defaults);

BEGIN {

    %tag_info_tags = (
		      track => 0,
		      title => 1,
		      artist => 2,
		      source => 3,  # originally 'album'
		      filesize => 6,
		      type => 7,
		      genre => 9,
		      duration => 14,

		      # The meanings of these tags are complete guesses.
		      bitrate => 10,
		      codec => 12,
		      '8' => 8
		      );
    %tag_info_defaults = (
			  genre => 'Unknown',
			  '8' => 'huh???',
			  bitrate => '??128',
			  type => 'tune'
			  );
}

sub list_tags {
    my ($q) = @_;

    my ($uri) = $ENV{'REQUEST_URI'};
    my ($trackid, %trackinfo);
    my ($answer);

    if ($uri =~ m-tags/([[:xdigit:]]+)-) {
	$trackid = $1;
    } else {
	$trackid = '';
    }

    %trackinfo = &get_one_track_info($trackid);

    $answer = '';
    foreach my $field (qw(artist bitrate codec duration track genre filesize 8 title source type)) {
	my ($value, $tagid);
	if (exists($trackinfo{$dbkeys{$field}})) {
	    $value = $trackinfo{$dbkeys{$field}};
	} elsif (exists $tag_info_defaults{$field}) {
	    $value = $tag_info_defaults{$field};
	} else {
	    next;
	}

	# Weird-ass binary format for returning tags of a track.
	$tagid = $tag_info_tags{$field};
	die unless defined $tagid;
	$value = substr($value, 0, 255) if length($value) > 255;
	$answer .= chr($tagid) . chr(length($value)) . $value;
    }
    $answer .= chr(0xFF); # sentinel

    print "Content-Type: application/octet-stream\n";
    printf "Content-Length: %d\n\n", length($answer);
    print $answer;
}

}


sub do_emit_music {
    my ($uri) = $ENV{'REQUEST_URI'};
    my (%trackinfo, $filename, $filesize, $mimetype);

    if ($uri =~ m-content/([[:xdigit:]]+)-) {
	%trackinfo = &get_one_track_info($1);
    }

    $filename = $trackinfo{$dbkeys{'path'}};
    $filesize = (defined $filename and -f $filename)? -s _ : 0;
    $mimetype = (exists $trackinfo{$dbkeys{'codec'}})?
	'audio/'.$trackinfo{$dbkeys{'codec'}} : 'application/octet-stream';

    print "Content-Length: $filesize\nContent-Type: $mimetype\n\n";
    system('cat', $filename) if $filesize > 0;
}
